/****************************************************************************
** Copyright (c) 2017, I.N.P.P. "DEMOKRITOS"
** All rights reserved.
**
**   Redistribution and use in source and binary forms, with or without
**   modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**      notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the I.N.P.P. "Demorkitos" nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL I.N.P.P. "DEMOKRITOS" BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
****************************************************************************/

#ifndef MAINWINDOW_H
   #define MAINWINDOW_H 1

   #include <iomanip>
   #include <fstream>
   #include <iostream>
   #include "Database.h"
   #include "ReactionWindow.h"
   #include "TargetWindow.h"

   #include <QMainWindow>
   #include <QtGui>
   #include <QStatusBar>
   #if (QT_VERSION < QT_VERSION_CHECK(4, 7, 0))
      #include <QTime>
      typedef QTime QElapsedTimer;
   #endif
   #if QT_VERSION >= 0x050000
      #include <QtWidgets>
   #endif


   QT_BEGIN_NAMESPACE
      class QAction;
      class QDialog;
      class QDialogButtonBox;
      class QGroupBox;
      class QLabel;
      class QLineEdit;
      class QMenu;
      class QMenuBar;
      class QPushButton;
      class QTextEdit;
      class QSpinBox;
      class QStatusBar;
      class QComboBox;
      class QMessageBox;
      class QIcon;
      class QSize;
      class QCheckBox;
      class QDoubleSpinBox;
      class QSpinBox;
      class QListWidget;
      class QListWidgetItem;
   QT_END_NAMESPACE

   class ReactionWindow;
   class TargetWindow;
   class stepDialog;

   class step
   {
   public:
       bool stepstate;
       double stepvalue;
   };
   class MainWindow : public QMainWindow
   {
      Q_OBJECT

      private:
         std::vector < isotope > Isotopes;
         std::vector < element > Elements;
         int Selected_Source;
         int Selected_Function;
         double zbl_stop(double);
         TargetWindow *target_window;
         ReactionWindow *react_window;
         void closeEvent(QCloseEvent *event);



      public:
         std::vector < target_element > Target;
         std::vector <reaction> Reaction_List;
         std::vector <reaction> Selected_reactions;
         MainWindow();

         bool include_strag;

         int program_choice;
         int r33_Z, r33_Mass;
         QFile r33filename;
         std::vector <double> E_cross, Cross_section, Yield_result;
         double MinStep, InitMinStep;
         double add_fract;

             QLabel *fit1_label[5];
             QLabel *init_element_yield_label[5];
             QDoubleSpinBox *init_element_yield_spin[5],  *init_element_energy_spin[5], *efficiency_spin[5];
             QDoubleSpinBox *element_yield_spin[5];



   public slots:
         void getvector(std::vector<reaction>Selected_reactions);
         void getTarget(std::vector < target_element > outTarget);
         void build_Target();
         void select_reaction();


      private slots:
         void fit();
         void show_selected();
         void open_r33();
         void save_results();
         void clear_results();
// Action Slots
         void check_srim();
         void check_zbl();
         void check_countsyield();
         void check_thickyield();
         void call_yield();
         void show_about();
         void create_reaction_file();
         void integ_step();

      protected:
   //      void closeEvent(QCloseEvent *event);

      private:
           void error_code(int,std::string);
           int warning_code(int);
   //      void setStatusBar();
           void createStatusBar();
           void createMenus();
           double energy_stop_calc(double);
           double interpol(int, double);
           double straggling(int,double,double);
           double SigmaBohr(int, double, double);
           void createLayout();
           void strag();
           void open_r33_file(QString);



      enum { NumGridRows = 3, NumButtons = 4 };

      QToolBar *fileToolBar;
      QString gen_text[6];
      QString line;
      QListWidget *targetList;
      QListWidgetItem *lopa;
      QTextEdit *targetEditor;
      QLabel *text_error[5], *isotope_label[10];

      QLabel *label_egamma, *bomb_energy, *charge_label, *yield_label, *meas_yield_label, *cutoff_energy,*eff_label;
      QPushButton *add_button, *remove_button, *fill_button;
      QPushButton *calc_exp_eff_button,*fit_repeat_button,*yield_button, *pige_button;
      QPushButton *back_button, *clear_button;
      QDialogButtonBox *buttonBox;
      QDoubleSpinBox *Abundance_spin[10],*bomb_energy_spin, *charge_spin;

      QComboBox *behav_combo,*element_combo;
      QMessageBox *msgBox;
      QIcon *func_icon,*func_icon1,*func_icon2;
      stepDialog *stepDial;
 //     ReactionWindow *react_window;
      QCheckBox *check_fit[5];
      QCheckBox *check_conf[5];

      QAction *ListAct;

      QPainter *painter;
      QTextEdit *ResultPane;
      QLabel *gen_label[6],*init_yield_label, *step_label, *statusLabel;
      QLabel *generic_label[5],*label_result[5];
      QPushButton  *calc_button ,  *save_button,  *open_button;
      QDoubleSpinBox *atomic_perc_spin,*init_yield_spin;

      std::vector< std::vector<double> > Tot_E_cross;
      std::vector< std::vector<double> > Tot_Cross_section;
      std::vector<QString> r33_info;  //info of the r33 files for the reaction list

      void createActions();

        QMenu *targetMenu;
        QMenu *optionsMenu;
        QMenu *stopMenu;
        QMenu *yieldMenu;
        QMenu *helpMenu;
        QMenu *ExtrasMenu;

        QAction *buildTargetAct;

        QAction *zblAct;
        QAction *srimAct;
        QAction *thickYieldAct;
        QAction *countsYieldAct;

        QAction *CreateReactFileAct;
        QAction *IntegStepAct;
        QAction *stragAct;
        QAction *ExtrasYieldAct;
        QAction *aboutAct;

        QAction *exitAct;
        QActionGroup *alignmentGroup;
        QAction *setLineSpacingAct;
        QAction *setParagraphSpacingAct;
        QLabel *infoLabel;
   };
   class stepDialog : public QDialog
    {
        Q_OBJECT

    public:
        stepDialog(QWidget *parent = 0);
        void SetData(double);
        step GetData();
        QCheckBox *step_check;
        step inter;


   private:

        QPushButton *closeButton;
        QLabel  *minStep_label, *step_label;
        QDoubleSpinBox *minStep_spin;


    private slots:
        void StepState();
        void StepClose();

   };


#endif
