/****************************************************************************
** Copyright (c) 2017, I.N.P.P. "DEMOKRITOS"
** All rights reserved.
**
**   Redistribution and use in source and binary forms, with or without
**   modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**      notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the I.N.P.P. "Demorkitos" nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL I.N.P.P. "DEMOKRITOS" BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
****************************************************************************/

#include "Database.h"

isotope :: isotope() {
}

isotope :: isotope(std::string name, int N, int Z, int MN, double m, double abd)
{
   Isotope_Name = name;
   Neutron_Number = N;
   Atomic_Number = Z;
   Mass_Number = MN;
   Mass = m;
   Abundance = abd;
}

element :: element() {
}

element :: element(std::string name, int Z, std::vector <int> N,
                   std::vector<int> MN,
                   std::vector<double> m, std::vector<double> abd)
{
   Element_Name = name;
   Atomic_Number = Z;

   Neutron_Number = N;
  Mass_Number = MN;
   Mass = m;
   Abundance = abd;
}

std::vector < isotope > create_isotopes() {
   int N, Z, M;
   std::vector < isotope > dummy;
   double mass,abun;
   std::string name,filepath;
   std::ifstream isotopefile;

   filepath = QApplication::applicationDirPath().toStdString().c_str();
   if(filepath=="/usr/local/bin") filepath="/usr/local/share/PiGreCo/data/masses.dat";
   else filepath.append("/data/masses.dat");
   isotopefile.open(filepath.c_str());
   if( !isotopefile.is_open() ) {
      QString fileName = QFileDialog::getOpenFileName(0,"Open file with masses");
      isotopefile.open(fileName.toUtf8().constData());
      if( !isotopefile.is_open() ) return dummy;
   }

   while( isotopefile>> N >> Z >> M >> name>>mass>>abun)
   {
       isotope temp(name,N,Z,M,mass,abun);
       dummy.push_back(temp);
   }
   isotopefile.close();
   return dummy;
}
std::vector < element > create_elements() {
//   int N1, Z1, M1;
   std::vector < isotope > vect_isot;
   std::vector < element > vect_elem;
 //  double mass1,abun1;
 //  std::string name1;
 //  std::ifstream isotopefile;
/*
   isotopefile.open("./data/masses.dat");
   if( !isotopefile.is_open() ) {
      QString fileName = QFileDialog::getOpenFileName(0,"Open file with masses");
      isotopefile.open(fileName.toUtf8().constData());
      if( !isotopefile.is_open() ) return vect_elem;
   }

   while( isotopefile>> N1 >> Z1 >> M1 >> name1>>mass1>>abun1 )
   {
       isotope temp_isot(name1,N1,Z1,M1,mass1,abun1);
       vect_isot.push_back(temp_isot);
   }
   isotopefile.close();
*/
   vect_isot = create_isotopes();
   int N_numb,i,j;

   std::vector < int > N;
   std::vector < int > A;
   std::vector < double > mass;
   std::vector < double > abun;
   std::string name;


   j=0;

   for(i=0;i<=vect_isot[vect_isot.size()-1].Atomic_Number;i++)
   {
   name = vect_isot[j].Isotope_Name;
       while ( i == vect_isot[j].Atomic_Number )
       {
          N_numb =  vect_isot[j].Neutron_Number;
          N.push_back(N_numb);
          mass.push_back(vect_isot[j].Mass);
          abun.push_back(vect_isot[j].Abundance);
           A.push_back(vect_isot[j].Mass_Number);
          j++;
       }
       if (N.size()!=0){
       element tmp_elem(name,i,N,A,mass,abun);
      vect_elem.push_back(tmp_elem);}
       N.clear();
       mass.clear();
       A.clear();
       abun.clear();
   }

   return vect_elem;
}

////////////////////////////////////Target constructor///////////////

target_element :: target_element() {}

target_element :: target_element(element el, double atper, int fit,std::vector<int>felem,std::vector<double>fratio)
{

    atom = el;
    atomic_percent = atper;
    fit_state = fit;
    linked_elem = felem;
    linked_ratio = fratio;
}

////////////////////////////////////Reaction constructor///////////////

reaction :: reaction() {}

reaction :: reaction(int Z, int A, float Energy, float th, int No, float Emin, float Emax,
                     std::string react,std::string file, std::string sour)
{
   Atomic_Number = Z;
   Mass_Number = A;
   Egamma = Energy;
   Theta = th;
   filename = file;
   E_min = Emin;
   E_max = Emax;
   No_of_points = No;
   Reaction_Name = react;
   Source = sour;
}
////////////////////////////////////Create Reaction List///////////////

std::vector < reaction > create_reaction_list() {
   int Z,A, No;
   std::vector < reaction > test;
   float E,th,Emin,Emax;
   std::string name,file_name, source,filepath;
   std::ifstream reactionfile;


   filepath = QApplication::applicationDirPath().toStdString().c_str();
   if(filepath=="/usr/local/bin") filepath=(QDir::homePath()+"/.R33Data/reactions.dat").toStdString().c_str();
   else filepath.append("/R33Data/reactions.dat");
   reactionfile.open(filepath.c_str());
   if( !reactionfile.is_open() ) {
      QString fileName = QFileDialog::getOpenFileName(0,"Open file with reactions");
      reactionfile.open(fileName.toUtf8().constData());
      if( !reactionfile.is_open() ) return test;
   }

   while( reactionfile>> Z >> A >> E >> th>> No >> Emin >> Emax >>name >> file_name )
   {   std::getline(reactionfile,source,'\n');

       reaction temp(Z, A, E,th,No, Emin ,Emax, name,file_name, source);
       test.push_back(temp);
   }
   reactionfile.close();
   return test;
}


isotope find_isotope_by_name(std::vector<isotope> initisotope, QString name)
{
    QStringList StringList = name.trimmed().split(QRegExp(":"));
    isotope dummy("None",0,0,0,0,0);
  //  qDebug() << myStringList.first() << myStringList.last();
    for(unsigned int i=0; i<initisotope.size()-1;i++)
    {
        if( QString::compare(StringList[0], QString::fromStdString(initisotope[i].Isotope_Name), Qt::CaseInsensitive) == 0)
            if(initisotope[i].Mass_Number == StringList[1].toInt())
                 return initisotope[i];
     }
    return dummy;
}

int find_isotope_by_Z(std::vector<isotope> initisotope, QString name)
{

    for(unsigned int i=0; i<initisotope.size()-1;i++)
    {
        if( QString::compare(name, QString::fromStdString(initisotope[i].Isotope_Name), Qt::CaseInsensitive) == 0)
            return initisotope[i].Atomic_Number;

    }
     return 0;
}


QString find_element_by_name(std::vector<element> initelement, QString name)
{
  //  QStringList StringList = name.trimmed().split(QRegExp(":"));
    QString dummy="0";
  //  qDebug() << myStringList.first() << myStringList.last();
    for(unsigned int i=0; i<initelement.size()-1;i++)
    {
        if( QString::compare(name, QString::fromStdString(initelement[i].Element_Name), Qt::CaseInsensitive) == 0)
            //if(initisotope[i].Mass_Number == StringList[1].toInt())
                 return QString::number(initelement[i].Atomic_Number);
     }
    return dummy;
}

