/****************************************************************************
** Copyright (c) 2017, I.N.P.P. "DEMOKRITOS"
** All rights reserved.
**
**   Redistribution and use in source and binary forms, with or without
**   modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**      notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the I.N.P.P. "Demorkitos" nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL I.N.P.P. "DEMOKRITOS" BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
****************************************************************************/

#include "ReactionWindow.h"
#include "Database.h"

//ReactionWindow:: ReactionWindow(){}

ReactionWindow :: ReactionWindow(MainWindow *main_window)
{

    QGridLayout *layout = new QGridLayout;
    QMessageBox msgBox;
    unsigned int element_found_count=0;
    local_main_window = main_window;    //// small trick to pass main_window to the slot
    QStringList test;
    setGeometry(geometry().x()+100, geometry().y()+100, 1000, 400);
    test << "Choose" <<" Gamma" << "Angle"<<"Energy region"<<"Source";

    for (int i=0;i<5;i++)
    {
      tabs[i] = new QWidget();
      tablayout[i] = new QGridLayout();
      react_table[i] = new QTableWidget();
      react_table[i]->setRowCount(50);
      react_table[i]->setColumnCount(5);
      react_table[i]->verticalHeader()->setVisible(false);
      react_table[i]->setColumnWidth(4,500);
      react_table[i]->setShowGrid(false);
      react_table[i]->setAlternatingRowColors(true);
      react_table[i]->setHorizontalHeaderLabels(test);

    }
    for(unsigned int i=0; i<5;i++)
    {
  //      main_window->fit1_label[i]->setText(" ");   ///// Delete fit element labels
  //      main_window->init_element_yield_spin[i]->setVisible(false);
  //      main_window->init_element_energy_spin[i]->setVisible(false);
  //      main_window->element_yield_spin[i]->setVisible(false);
  //      main_window->efficiency_spin[i]->setVisible(false);
    }

    element_tab = new QTabWidget();

    for(int i=0;i<50;i++)
         cross_check[i] = new QCheckBox();

    choose_button = new QPushButton;
    choose_button->setText("OK");
    connect(choose_button, SIGNAL(released()), this, SLOT(choose_cs()));
    layout->addWidget(choose_button,40,0);
  //    qDebug() <<main_window->Target.size()<<" "<<main_window->Reaction_List.size();

    int k=0;
    int l=0;
    unsigned int fit_element_count =0;

    for (unsigned int i=0; i< main_window->Target.size();i++)
   {
      bool element_found = false;
      if (main_window->Target[i].fit_state == 1 ) // If fit is selected for the element
     {

      for(unsigned int j=0; j< main_window->Reaction_List.size();j++ )
      {

       if(main_window->Target[i].atom.Atomic_Number  == main_window->Reaction_List[j].Atomic_Number)
       {

        element_found = true;
        react_table[fit_element_count]->setItem(k,4,new QTableWidgetItem(QString(main_window->Reaction_List[j].Reaction_Name.c_str())+
                                                         " Source: "+QString(main_window->Reaction_List[j].Source.c_str())));
        react_table[fit_element_count]->setCellWidget(k,0,cross_check[l]);
        react_table[fit_element_count]->setItem(k,1,new QTableWidgetItem(QString::number(main_window->Reaction_List[j].Egamma)));
        react_table[fit_element_count]->setItem(k,2,new QTableWidgetItem(QString::number(main_window->Reaction_List[j].Theta)));
        react_table[fit_element_count]->setItem(k,3,new QTableWidgetItem(QString::number(main_window->Reaction_List[j].E_min)
                                                         +" - " + QString::number(main_window->Reaction_List[j].E_max)));

        Possible_reactions.push_back(main_window->Reaction_List[j]);

        react_table[fit_element_count]->item(k,1)->setFlags(react_table[fit_element_count]->item(k,1)->flags() ^ Qt::ItemIsEditable);
        react_table[fit_element_count]->item(k,2)->setFlags(react_table[fit_element_count]->item(k,2)->flags() ^ Qt::ItemIsEditable);
        react_table[fit_element_count]->item(k,4)->setFlags(react_table[fit_element_count]->item(k,4)->flags() ^ Qt::ItemIsEditable);

        k++;  //// Number of reactions per fit element
        l++;  //// Number of Checkboxes
        }
      }
      for (int kk=k; kk<50; kk++) react_table[fit_element_count]->hideRow(kk);
      react_table[fit_element_count]->sortItems(1,Qt::AscendingOrder);
      tablayout[fit_element_count]->addWidget(react_table[fit_element_count],0,0,50,50);
     if(element_found)
     {
      element_tab->addTab(tabs[fit_element_count],QString(main_window->Target[i].atom.Element_Name.c_str()));
  //    qDebug()<<"lola "<<element_found_count;
    //  main_window->fit_label[element_found_count]->setText(main_window->Target[i].atom.Element_Name.c_str());

      element_found_count++;                                ///// just to count elements for fit

     }
     else
     {
         msgBox.setWindowTitle("Error");
         msgBox.setText("No Available Data for " + tr(main_window->Target[i].atom.Element_Name.c_str()) +"\n" +
                        "Please change the fit behavior");
         msgBox.setIcon(QMessageBox::Critical);
         msgBox.exec();
     }
          tabs[fit_element_count]->setLayout(tablayout[fit_element_count]);
            k=0;
            fit_element_count++;

    }  // end if
 //     tabs[i]->addW
   }
/////  Check the boxes of previously selected reactions /////////

    for( unsigned int i=0; i < Possible_reactions.size();i++)
    {
        for(unsigned int j=0; j< main_window->Selected_reactions.size();j++ )
        {
        if(main_window->Selected_reactions[j].filename == Possible_reactions[i].filename)
            cross_check[i]->setChecked(true);
        }
    }



    layout->addWidget(element_tab,0,0);
   QWidget *window = new QWidget();
    window->setLayout(layout);
    setCentralWidget(window);
    setWindowTitle(tr("Reaction Chooser"));
}

void ReactionWindow::choose_cs(){

    QMessageBox msgBox;

////// Copy Possible reactions to Temp reactions and erase those which are not checked///////

    Temp_reactions = Possible_reactions;
    for ( int i=Temp_reactions.size()-1; i >= 0 ; i--)
    {
       if( cross_check[i]->isChecked()==false )
             Temp_reactions.erase(Temp_reactions.begin()+i);
    }

////////  Loop over all the reactions to check if there is overlapping ////////

    for ( unsigned int i=0; i < Temp_reactions.size() ; i++)
    {
       for (unsigned int j=i; j < Temp_reactions.size(); j++ )
       {
       if( i!=j && Temp_reactions[i].Atomic_Number == Temp_reactions[j].Atomic_Number )
         {
          msgBox.setWindowTitle("Error");
          msgBox.setText("Multiple cross sections selected. Please choose only one");
          msgBox.setIcon(QMessageBox::Critical);
          msgBox.exec();
          return;
         }
       }

       for ( unsigned int i=0; i < Temp_reactions.size() ; i++)
       {
          local_main_window->fit1_label[i]->setText("Egamma = "+QString::number(Temp_reactions[i].Egamma));
    //      local_main_window->init_element_energy_spin[i]->setVisible(true);
    //      local_main_window->init_element_yield_spin[i]->setVisible(true);
    //      local_main_window->element_yield_spin[i]->setVisible(true);
    //      local_main_window->efficiency_spin[i]->setVisible(true);
          local_main_window->init_element_energy_spin[i]->setValue(Temp_reactions[i].E_min);
//          qDebug()<<Temp_reactions[i].E_min;
       }
    }





   this->close();
}
void ReactionWindow::closeEvent(QCloseEvent *event)
{
 event->accept();
 emit sendreact(Temp_reactions);
}
void ReactionWindow::checkbox_check()
{
}
