/****************************************************************************
** Copyright (c) 2017, I.N.P.P. "DEMOKRITOS"
** All rights reserved.
**
**   Redistribution and use in source and binary forms, with or without
**   modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**      notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the I.N.P.P. "Demorkitos" nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL I.N.P.P. "DEMOKRITOS" BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
****************************************************************************/

#include "mainwindow.h"
#include "ReactionWindow.h"
#include <string>

MainWindow::MainWindow()
{
    // Initializing files
    Reaction_List = create_reaction_list();
    Elements = create_elements();
    target_window = new TargetWindow(this);

    stepDial = new stepDialog(this);
    stepDial->setFixedHeight(150);
    stepDial->setFixedWidth(250);
  //          createStatusBar();

     createLayout();
}


void MainWindow::createLayout()
 {

    QGridLayout *layout = new QGridLayout;

    open_button = new QPushButton();
    open_button->setDisabled(false);
    open_button->setFixedSize(120,30);
    open_button->setSizePolicy(QSizePolicy::MinimumExpanding,
                        QSizePolicy::MinimumExpanding);
    open_button->setText(tr("Select Reaction"));
    connect(open_button, SIGNAL(released()), this, SLOT(select_reaction()));

    layout->addWidget(open_button,1,2);
    layout->setAlignment(open_button,Qt::AlignCenter);

    clear_button = new QPushButton(tr("Clear Results"));
    clear_button->setDisabled(false);
    connect(clear_button, SIGNAL(released()), this, SLOT(clear_results()));
    layout->addWidget(clear_button,13,5);

    save_button = new QPushButton(tr("Save Results"));
    save_button->setDisabled(false);
    connect(save_button, SIGNAL(released()), this, SLOT(save_results()));
    layout->addWidget(save_button,13,6);

    calc_button = new QPushButton(tr("Fit"));
    calc_button->setDisabled(true);
    connect(calc_button, SIGNAL(released()), this, SLOT(fit()));
    layout->addWidget(calc_button,1,5);


    ResultPane = new QTextEdit;
    ResultPane->setMinimumHeight(500);
    ResultPane->setMaximumHeight(500);
    ResultPane->setPlainText(tr(""));
    ResultPane->setReadOnly(true);
    this->resize(1000,600);


    bomb_energy = new QLabel();
    bomb_energy->setText("Bombarding Energy (keV)");
    layout->addWidget(bomb_energy,2,0);

    bomb_energy_spin = new QDoubleSpinBox;
    bomb_energy_spin->setRange(0,100000);
    layout->addWidget(bomb_energy_spin,2,1);

    charge_label = new QLabel();
    charge_label->setText("Charge (uCb)");
    layout->addWidget(charge_label,2,2);

    charge_spin = new QDoubleSpinBox;
    charge_spin->setRange(0,100000);
    layout->addWidget(charge_spin,2,3);

    eff_label = new QLabel();
    eff_label->setText("Efficiency");
    layout->addWidget(eff_label,3,4);

    meas_yield_label = new QLabel();
    meas_yield_label->setText("Measured Yield (counts)");
    layout->addWidget(meas_yield_label,3,3);

    yield_label = new QLabel();
    yield_label->setText("Initial Yield (N/uCb/sr)");
    layout->addWidget(yield_label,3,2);

    cutoff_energy = new QLabel();
    cutoff_energy->setText("Cuttof Energy (keV)");
    layout->addWidget(cutoff_energy,3,1);

    for (unsigned int i=0; i<2;i++)
     {
     generic_label[i] = new QLabel();
     QFont font = generic_label[i]->font();
     font.setPointSize(16);
     font.setBold(true);
     generic_label[i]->setFont(font);
     generic_label[i]->setHidden(false);  //react_window = new ReactionWindow(this);
     generic_label[i]->setAlignment(Qt::AlignCenter);
   }


    for(int i=0;i<5;i++){
     //  check_fit[i] = new QCheckBox();
       fit1_label[i] = new QLabel();

       init_element_yield_spin[i] = new QDoubleSpinBox;
       init_element_energy_spin[i] = new QDoubleSpinBox;
       element_yield_spin[i] = new QDoubleSpinBox;
       efficiency_spin[i] = new QDoubleSpinBox;

       init_element_yield_spin[i]->setVisible(false);
       init_element_yield_spin[i]->setRange(0,10000000);

       init_element_energy_spin[i]->setVisible(false);
       init_element_energy_spin[i]->setRange(0,100000);

       element_yield_spin[i]->setVisible(false);
       element_yield_spin[i]->setRange(0,10000000);

       efficiency_spin[i]->setVisible(false);
       efficiency_spin[i]->setDecimals(5);
        efficiency_spin[i]->setRange(0,1);

       layout->addWidget(fit1_label[i],4+i,0);

       layout->addWidget(init_element_energy_spin[i],4+i,1);
       layout->addWidget(init_element_yield_spin[i],4+i,2);
       layout->addWidget(element_yield_spin[i],4+i,3);
       layout->addWidget(efficiency_spin[i],4+i,4);

}
       generic_label[0]->setText("Calculation Parameters");
       layout->addWidget(generic_label[0],0,0,1,5);

       generic_label[1]->setText("Fitting");
       layout->addWidget(generic_label[1],0,3,1,5);
 //   }

    layout->addWidget(ResultPane, 2, 5, 7, 3);


     QWidget *window = new QWidget();
     window->setLayout(layout);
     setCentralWidget(window);
     menuBar()->setNativeMenuBar(false);
    createStatusBar();
     createActions();
     createMenus();
     const QChar MathSymbolPi(0x03A0);
     setWindowTitle(MathSymbolPi + tr(" - PiGreCo v1.0"));


}

void MainWindow::integ_step()
{
    stepDial->show();
}

void MainWindow::strag()
{
    if(stragAct->isChecked()==true)
        include_strag = true;
    else
        include_strag = false;
}

void MainWindow::createActions()
{
    buildTargetAct = new QAction(tr("&Target"), this);
    buildTargetAct->setShortcut(tr("Ctrl+T"));
    buildTargetAct->setStatusTip(tr("Build target"));
    connect(buildTargetAct, SIGNAL(triggered()), this, SLOT(build_Target()));

    zblAct = new QAction(tr("&ZBL"), this);
    zblAct->setCheckable(true);
    zblAct->setChecked(true);
    zblAct->setStatusTip(tr("ZBL stopping power"));
    connect(zblAct, SIGNAL(triggered()), this, SLOT(check_zbl()));

    srimAct = new QAction(tr("SRIM"), this);
    srimAct->setCheckable(true);
    srimAct->setStatusTip(tr("Exit the application"));
    connect(srimAct, SIGNAL(triggered()), this, SLOT(check_srim()));

    stragAct = new QAction(tr("S&traggling"), this);
    stragAct->setStatusTip(tr("Include straggling calculation"));
    stragAct->setCheckable(true);

    zblAct = new QAction(tr("&ZBL"), this);
    zblAct->setCheckable(true);
    zblAct->setChecked(true);
    zblAct->setStatusTip(tr("ZBL stopping power"));
    connect(zblAct, SIGNAL(triggered()), this, SLOT(check_zbl()));

    srimAct = new QAction(tr("SRIM"), this);
    srimAct->setCheckable(true);
    srimAct->setStatusTip(tr("SRIM stopping power"));
    connect(srimAct, SIGNAL(triggered()), this, SLOT(check_srim()));

    thickYieldAct = new QAction(tr("&Thick target"), this);
    thickYieldAct->setCheckable(true);
    thickYieldAct->setChecked(true);
    thickYieldAct->setStatusTip(tr("Enter Thick target yield as Initial yield"));
    connect(thickYieldAct, SIGNAL(triggered()), this, SLOT(check_thickyield()));

    countsYieldAct = new QAction(tr("&Counts"), this);
    countsYieldAct->setCheckable(true);
    countsYieldAct->setChecked(false);
    countsYieldAct->setStatusTip(tr("Enter Counts as Initial yield"));
    connect(countsYieldAct, SIGNAL(triggered()), this, SLOT(check_countsyield()));

    stragAct = new QAction(tr("S&traggling"), this);
    stragAct->setStatusTip(tr("Include straggling calculation"));
    stragAct->setCheckable(true);

    CreateReactFileAct = new QAction(tr("&Create reaction List"), this);
    CreateReactFileAct->setDisabled(false);
    connect(CreateReactFileAct,SIGNAL(triggered()),this,SLOT(create_reaction_file()));


    IntegStepAct = new QAction(tr("&Integration Step"), this);
    connect(IntegStepAct, SIGNAL(triggered()), this, SLOT(integ_step()));

    ExtrasYieldAct = new QAction(tr("&Yield calc"), this);
    connect(ExtrasYieldAct, SIGNAL(triggered()), this, SLOT(call_yield()));

    aboutAct = new QAction(tr("A&bout"), this);
    aboutAct->setStatusTip(tr("About the program"));
    connect(aboutAct, SIGNAL(triggered()), this, SLOT(show_about()));

}

void MainWindow::createMenus()
{

    menuBar()->addAction(buildTargetAct);


/*     fileMenu->addAction(saveAct);
    fileMenu->addAction(printAct);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);
*/
    optionsMenu = menuBar()->addMenu(tr("&Options"));

    optionsMenu->addAction(CreateReactFileAct);
    optionsMenu->addAction(IntegStepAct);

    stopMenu = optionsMenu->addMenu(tr("&Stopping Power"));
    stopMenu->addAction(zblAct);
    stopMenu->addAction(srimAct);

    yieldMenu = optionsMenu->addMenu(tr("Initial Y&ield"));
    yieldMenu->addAction(thickYieldAct);
    yieldMenu->addAction(countsYieldAct);

    optionsMenu->addAction(stragAct);

 //   ExtrasMenu = menuBar()->addMenu(tr("&Extras"));
 //   ExtrasMenu->addAction(ExtrasYieldAct);

    helpMenu= menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAct);

  /*  filemenu=menuBar()->addMenu(tr("&File"));
    ListAct = new QAction(tr("&New"), this);help
    filemenu->addAction(ListAct);
    Optionmenu=menuBar()->addMenu(tr("&Options"));
    ListAct = new QAction(tr("&Create reaction List"), this);
    Optionmenu->addAction(ListAct);
    connect(ListAct,SIGNAL(triggered()),this,SLOT(create_reaction_file()));

    editMenu->addAction(undoAct);
    editMenu->addAction(redoAct);
    editMenu->addSeparator();
    editMenu->addAction(cutAct);
    editMenu->addAction(copyAct);
    editMenu->addAction(pasteAct);
    editMenu->addSeparator();

    helpMenu = menuBar()->addMenu(tr("&Help"));
    helpMenu->addAction(aboutAct);
    helpMenu->addAction(aboutQtAct);

    formatMenu->addSeparator()->setText(tr("Alignment"));
    formatMenu->addAction(leftAlignAct);
    formatMenu->addAction(rightAlignAct);
    formatMenu->addAction(justifyAct);
    formatMenu->addAction(centerAct);
    formatMenu->addSeparator();
    formatMenu->addAction(setLineSpacingAct);
    formatMenu->addAction(setParagraphSpacingAct); */
}
//////////////////////////////////////////////////////////////////

    void MainWindow::show_about()
    {
        QMessageBox msgBox;
        msgBox.setWindowTitle("About");
        msgBox.setIcon(QMessageBox::Information);
        msgBox.setText("PiGreCo program ver. 1.0 \n A program for PIGE calculation \n For info please contact \n K. Preketes Sigalas \n preketes@inp.demokritos.gr");
         msgBox.exec();
    }



    void MainWindow::call_yield()
    {
       if( QProcess::execute("/usr/bin/nedit") < 0 )
        return;

    }


    void MainWindow::getTarget(std::vector < target_element > outTarget)
    {
       Target = outTarget;

 //      for(unsigned int i=0;i<Target.size();i++)
 //      {
 //          if(Target[i].atom.Element_Name == "") qDebug()<<"ouch";
 //           qDebug() << outTarget[i].atom.Element_Name.c_str();
 //      }
    }

//////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////

    void MainWindow::check_zbl()
    {
        srimAct->setChecked(false);
        zblAct->setChecked(true);
    }

//////////////////////////////////////////////////////////////////

    void MainWindow::check_srim()
    {
            srimAct->setChecked(true);
            zblAct->setChecked(false);
    }

//////////////////////////////////////////////////////////////////


        void MainWindow::check_countsyield()
        {
            thickYieldAct->setChecked(false);
            countsYieldAct->setChecked(true);
            yield_label->setText("Initial Yield (counts)");
        }

 //////////////////////////////////////////////////////////////////

        void MainWindow::check_thickyield()
        {
                thickYieldAct->setChecked(true);
                countsYieldAct->setChecked(false);
                yield_label->setText("Initial Yield (N/uCb/sr)");
        }

    //////////////////////////////////////////////////////////////////

void MainWindow::getvector(std::vector<reaction> Sel)
{
    Selected_reactions = Sel;
    if(Selected_reactions.size() == 0)
    {
        error_code(3,"1");
        calc_button->setEnabled(false);

        return;
    }
    calc_button->setEnabled(true);
  //  for(unsigned int i=0;i<Selected_reactions.size();i++)
  //  qDebug() << QString(Selected_reactions[i].filename.c_str());
}

/////////////// Open New Window to select cross section /////////////

void MainWindow::select_reaction() {
    bool free_exist = false;
    bool fit_exist = false;
    double sum=0;
    if(Target.size()==0)
    {
        error_code(1,"");
        return;
    }
    if(Target.size() < 2)
    {
       error_code(4,"1");
       return;
    }
    for(unsigned int k=0;k<Target.size();k++)
    {
        if( Target[k].fit_state == 0) free_exist = true;
        if( Target[k].fit_state == 1) fit_exist = true;
    }
    if ( !free_exist || !fit_exist)
    {
       error_code(5,"1");
       return;
    }

    for (unsigned int i=0; i< Target.size();i++)
   {
      bool element_found = false;
      if (Target[i].fit_state == 1 ) // If fit is selected for the element
     {

      for(unsigned int j=0; j< Reaction_List.size();j++ )
          if(Target[i].atom.Atomic_Number  == Reaction_List[j].Atomic_Number)
               element_found = true;

      if(!element_found)
      {
          error_code(6,Target[i].atom.Element_Name);
          return;
      }
       }
    }
    for (unsigned int i=0; i< Target.size();i++)
    {
        sum += Target[i].atomic_percent;
    }
    if(sum != 1)
     {
      error_code(2,"1");
      return;
    }
 react_window = new ReactionWindow(this);
    react_window->show();
    QObject::connect(react_window,SIGNAL(sendreact(std::vector <reaction>)),
                                this,SLOT(getvector(std::vector <reaction>)));

}

////// Overide of the < operator so I can sort the target elements
///    according to their fit_state

bool operator<(const target_element &s1, const target_element &s2)
{
    if(s1.fit_state < s2.fit_state)
        return true;
    else
        return false;
}


////////////////////////////////////////////////////////////////////
/// \brief MainWindow::error_code
/// \param code The actual error code
/// \param i  Only used in case where code=6. Identifies the element
///////////////////////////////////////////////////


void MainWindow::error_code(int code, std::string name)
{
    QMessageBox msgBox;
    msgBox.setWindowTitle("Error");
    msgBox.setIcon(QMessageBox::Critical);

    switch (code) {
    case 1:
        msgBox.setText("Target has empty elements or Target window still open");
        break;
    case 2:
        msgBox.setText("Sum of atomic percentages should equal 1");
        break;
    case 3:
        msgBox.setText("You must select a cross section");
        break;
    case 4:
        msgBox.setText("Target should at least contain two elements");
        break;
    case 5:
        msgBox.setText("Target should contain at least one free and one fit element");
        break;
    case 6:
        msgBox.setText("No Available Data for " + QString(name.c_str()) +"\n" +
                                "Please change the fit behavior");
        break;
    case 7:
        msgBox.setText("You have selected fewer cross section data \n  than fit elements");
        break;
    case 8:
        msgBox.setText("Initial yield of " + QString(name.c_str()) +"\n" +
                                "greater than measured yield");
        break;
    case 9:
        msgBox.setText("Bombarding energy outside r33 range (" + QString(name.c_str()) +")");
        break;
    case 10:
        msgBox.setText("Cutoff energy outside r33 range (" + QString(name.c_str()) + ")");
        break;
    case 11:
        msgBox.setText("Bombarding energy less than Cutoff energy (" + QString(name.c_str()) + ")");
        break;
    case 12:
        msgBox.setText("Please enter efficiency (" + QString(name.c_str()) + ")");
        break;
    case 13:
        msgBox.setText("Charge value not valid");
        break;
    case 14:
        msgBox.setText("Invalid concentration in the solution (" + QString(name.c_str()) + ")");
        break;
    case 15:
        msgBox.setText("The r33 file doesnot contain cross sections");
        break;
    default:
        break;
    }
    msgBox.exec();

}
//////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////
/// \brief MainWindow::warning_code
/// \param code The actual error code
/// \param i  Only used in case where code=6. Identifies the element
///////////////////////////////////////////////////


int MainWindow::warning_code(int code)
{
    QMessageBox msgBox;
    msgBox.setWindowTitle("Warning");
    msgBox.setIcon(QMessageBox::Warning);

    switch (code) {
    case 1:
        msgBox.setText("Enabling srim may take some time");
        msgBox.setInformativeText("Do you want to procceed with the fit?");
        msgBox.setStandardButtons(QMessageBox::Ok|QMessageBox::Cancel);
        break;
    case 2:
        msgBox.setText("Enabling straggling may take some time");
        msgBox.setInformativeText("Do you want to procceed with the fit?");
        msgBox.setStandardButtons(QMessageBox::Ok|QMessageBox::Cancel);
        break;
    case 3:
        msgBox.setText("Setting integration step less than 5 keV may take some time");
        msgBox.setInformativeText("Do you want to procceed with the fit?");
        msgBox.setStandardButtons(QMessageBox::Ok|QMessageBox::Cancel);
        break;
     default:
        break;
    }
    return msgBox.exec();

}
//////////////////////////////////////////////////////

void MainWindow::fit(){
    double E_current, Cross_current, atom_abun[5], atom_perc[5],init_perc[20],s0,s, link_ratio[10];
    unsigned int No_free = 0,No_fit = 0, No_fixed = 0;
    std::string filepath;
    isotope fit_isotope[2];
    QString dummy;
    QStringList files;
    QElapsedTimer timer;
    int decis;

    ///////////  ///////////////
    /// Sorting and counting the target elements order (Free / Fit / Fixed )
    ////////////////////////

    sort(Target.begin(),Target.end());

        for(unsigned int i = 0; i < Target.size(); i++)
        {
            if ( Target.at(i).fit_state == 0 ) No_free++;
            if ( Target.at(i).fit_state == 1 ) No_fit++;
            if ( Target.at(i).fit_state == 2 ) No_fixed++;
        }




        filepath = QApplication::applicationDirPath().toStdString().c_str();
   if(filepath=="/usr/local/bin") filepath=(QDir::homePath()+"/.R33Data/").toStdString().c_str();
   else filepath.append("/R33Data/");

    for(unsigned int j=0;j< Target.size();j++)
    {
    init_perc[j] = Target[j].atomic_percent;
  //  qDebug() << "Target " << Target[j].atom.Element_Name.c_str() << Target[j].linked_elem[0]<<Target[j].linked_ratio[0];

    }


    for(unsigned int i=0; i < Selected_reactions.size();i++)
    {
        files.append(QString(((filepath + Selected_reactions[i].filename).c_str())));
   //   qDebug() << "Skotodini" << QString(Selected_reactions[i].filename.c_str()) << Selected_reactions.size();
        for(unsigned int j=0;j< Target.size();j++)
        {
            if (Selected_reactions[i].Atomic_Number==Target[j].atom.Atomic_Number)
            {
                atom_perc[i] = Target[j].atomic_percent;
                link_ratio[i] = Target[j].linked_ratio[0];
                for(unsigned int k = 0; k < Target[j].atom.Abundance.size();k++)
                {
                    fit_isotope[i].Isotope_Name = Target[j].atom.Element_Name;
                    if (Selected_reactions[i].Mass_Number == Target[j].atom.Mass_Number[k])
                         fit_isotope[i].Abundance = Target[j].atom.Abundance[k];
                }
            }

        }
     }
    for(unsigned int i=0; i < Selected_reactions.size();i++)
        atom_abun[i]=fit_isotope[i].Abundance;

   if (Selected_reactions.size() != No_fit) {error_code(7,""); return;}

    double yield[5]={0,0,0,0,0};
    double dE;
    E_current = bomb_energy_spin->value();
    float dif[5] = {100,100,100,100,100};   ////  initial difference (whatever)

    double user_yield[5], check_perc[10];
    double init_yield[5];
    unsigned int iter = 0;
    unsigned int fitelem = 0;
    bool tot_conv = false;
    bool error_flag=false;
    bool conv[5] = {false}, linked[10]={false};


    if(stepDial->GetData().stepstate == false)
        dE = stepDial->GetData().stepvalue;
    else
        dE = 5;


    ///////////////////////////////////////////////////////////////////////////////////////////

   ///Warnings for Srim, Integration and Straggling
   ///
   //////////////////////////////////

       if(srimAct->isChecked()==true)
           decis = warning_code(1);
        if(stragAct->isChecked()==true)
           decis = warning_code(2);
        if(dE<5)
            decis = warning_code(3);

        qDebug()<<decis;
        switch(decis)
        {
        case QMessageBox::Ok:
            break;
        case QMessageBox::Cancel:
            return;
            break;
        default:
            break;
        }
    for (unsigned int iter=0; iter < Selected_reactions.size(); iter++)
    {
        open_r33_file(files[iter]);                    ///// Reading the cross sections
        Tot_E_cross.push_back(E_cross);                ////// Vector of vector of energies for each element
        Tot_Cross_section.push_back(Cross_section);    ///// Vector of vector of cs for each element


        if (efficiency_spin[iter]->value() <= 0) {error_code(12,fit_isotope[iter].Isotope_Name); return;}  ///// Check for valid efficiency
        if (charge_spin->value() <= 0) {error_code(13,""); return;}             ///// Check for valid charge

         user_yield[iter] = element_yield_spin[iter]->value()/(efficiency_spin[iter]->value()*4*3.141592)/charge_spin->value();  ////// input yield



         ////// Check units of initial yield and change to N/uCb/srQString::numbe
         if (thickYieldAct->isChecked()==true)
                 init_yield[iter] = init_element_yield_spin[iter]->value();
         else
                 init_yield[iter] = init_element_yield_spin[iter]->value()/(efficiency_spin[iter]->value()*4*3.141592)/charge_spin->value();

        if (user_yield[iter] <= init_yield[iter]) {error_code(8,fit_isotope[iter].Isotope_Name);return;} //// Check for yields

        if (E_current <= E_cross[0] || E_current > E_cross[E_cross.size()-1] ) {error_code(9,fit_isotope[iter].Isotope_Name); return;} //// Bombarding in r33
        if (init_element_energy_spin[iter]->value() < E_cross[0] || init_element_energy_spin[iter] ->value() > E_cross[E_cross.size()-1] )
                                                                                   {error_code(10,fit_isotope[iter].Isotope_Name); return;} ////cutoff in r33
        if (E_current <= init_element_energy_spin[iter]->value() ) {error_code(11,fit_isotope[iter].Isotope_Name); return;} ///// Bombarding less than cutoff
    }

    for (unsigned int k=0; k < No_fit; k++)
        for(unsigned int kk=0; kk<No_free; kk++)
            if((unsigned int)Target[No_free+k].linked_elem[0] == kk)
                linked[kk] =true;
    int No_linked = 0;
    for (unsigned int k=0; k < No_fit; k++)
        if(linked[k] == true)
            No_linked++;

   timer.start();
    while( !tot_conv && iter != 50 && !error_flag )
   {

        for(fitelem=0; fitelem < No_fit; fitelem++)
        {
            if(!conv[fitelem])
            {
               yield[fitelem]=0;
               s0 = 1;                                 ///// Initial energy ripple

       while( E_current > init_element_energy_spin[fitelem]->value())
       {
           if( stragAct->isChecked() == false)
           {
              Cross_current = interpol(fitelem,E_current);
              yield[fitelem] += Cross_current/energy_stop_calc(E_current)*dE;
           }
           else
           {
            s =SigmaBohr(dE, E_current, s0);
            yield[fitelem] += straggling(fitelem,E_current,s)/energy_stop_calc(E_current)*dE;
            s0 = s;
            }
            E_current -= dE;
       }
       yield[fitelem] = yield[fitelem] *1e-27*1e15*6.24E12*1000*atom_perc[fitelem]*(atom_abun[fitelem]/100); /// Yield calculation
       yield[fitelem] += init_yield[fitelem]*atom_perc[fitelem];

       dif[fitelem] = yield[fitelem] / user_yield[fitelem]; //// Ratio of calculated yield (yield) with input yield (user_yield)
       atom_perc[fitelem] = atom_perc[fitelem]/dif[fitelem];                              ///// changing the percentage by the dif
 //      link_perc[fitelem] = atom_perc[fitelem]*link_ratio[fitelem];
  //     double avail_perc = Target[No_free+fitelem].atomic_percent - atom_perc[fitelem] - link_perc[fitelem];   //// Available percentage (pos or neg)
       double avail_perc = Target[No_free+fitelem].atomic_percent - atom_perc[fitelem];   //// Available percentage (pos or neg)

       Target[No_free+fitelem].atomic_percent = atom_perc[fitelem];                      ///// assigning the new percentage to the fit element
       avail_perc = avail_perc / No_free;                                                ///// dividing the available percentage to free elements

       for (unsigned int k=0; k < No_free; k++)
       {
              Target[k].atomic_percent += avail_perc;
              check_perc[k] =0;
       }


 //      qDebug()<<"------------------------";
 //      qDebug()<<"1"<<avail_perc;

       for (unsigned int k=0; k < No_fit; k++)
           for(unsigned int kk=0; kk<No_free; kk++)
               if(linked[kk] == true)
                   check_perc[kk] += Target[No_free+k].atomic_percent*link_ratio[k];

       for(unsigned int kk=0; kk<No_free; kk++)
  //                qDebug()<<"2"<< check_perc[kk];
       avail_perc=0;
       for (unsigned int k=0; k < No_free; k++)
           if( linked[k]==true && check_perc[k] > Target[k].atomic_percent)
           {
               avail_perc += (check_perc[k] - Target[k].atomic_percent);
               Target[k].atomic_percent= check_perc[k];
           }
       avail_perc = avail_perc/(No_free-No_linked);

       for(unsigned int kk=0; kk<Target.size(); kk++)
  //                qDebug()<<"3"<< Target[kk].atom.Element_Name.c_str()<<" "<<Target[kk].atomic_percent;

       for (unsigned int k=0; k < No_free; k++)
           if( linked[k]!=true )
               Target[k].atomic_percent -= avail_perc;




        E_current = bomb_energy_spin->value();
        }  ///// end if

        if(dif[fitelem] < 1.001 && dif[fitelem] > 0.999)
          {
            conv[fitelem] = true;
          }
       } ///// end for
       iter++;

        ResultPane->append("Number of iterations: " +  QString::number(iter));
        for (unsigned k=0; k < Target.size(); k++)
            ResultPane->append(QString(Target.at(k).atom.Element_Name.c_str()) +" "+ QString::number(Target.at(k).atomic_percent));
        ResultPane->append("-----------------------------------");
        ResultPane->repaint();

        tot_conv = conv[0];
        for(unsigned int jj=1;jj<No_fit;jj++)
           tot_conv = tot_conv && conv[jj] ;

        for(unsigned int kk=0; kk<Target.size(); kk++)
                   if (Target[kk].atomic_percent >= 1 || Target[kk].atomic_percent <= 0)
                   {
                       error_code(14,Target[kk].atom.Element_Name);
                       error_flag = true;
                       ResultPane->clear();
  //                     return;
                   }
        dummy = QString::number(timer.elapsed());
   //     qDebug() <<"ola "<< timer.elapsed();

      statusLabel->setText("Time elapsed: "+dummy+" millisec");
   }

    for (unsigned int k=0; k < Target.size(); k++)
    {
        Target.at(k).atomic_percent = init_perc[k];
//        qDebug()<<"hola"<<k<< " "<< init_perc[k];
//        if( Target.at(k).atomic_percent <= 0 ) error_code(14,Target.at(k).Element_Name);
    }
ResultPane->append("Done!");
}

//////////////////////////////////////////////////////////////////

double MainWindow::interpol(int elem, double energy)
{
    double cs=0;
    for (unsigned int i=0; i < Tot_E_cross[elem].size() -1 ; i++ )
    if (energy > Tot_E_cross[elem][i] && energy <= Tot_E_cross[elem][i+1] )
    {
        cs = Tot_Cross_section[elem][i] +( Tot_Cross_section[elem][i+1]- Tot_Cross_section[elem][i])
              *((energy-Tot_E_cross[elem][i])/(Tot_E_cross[elem][i+1]-Tot_E_cross[elem][i]));
        break;
    }
    return cs;
}

//////////////////////////////////////////////////////////////////

double MainWindow::SigmaBohr(int dE, double energy, double sigma)
{
    double snon2, sbohr2 = 0;
    for(unsigned int k=0; k <= Target.size()-1;k++)
    {
       sbohr2 += Target[k].atomic_percent * 0.26 * Target[k].atom.Atomic_Number * (dE  / energy_stop_calc(energy));    ////// Initial Bohr stragling
    }
    snon2 = pow(energy_stop_calc(energy)/energy_stop_calc(energy-dE),2)*pow(sigma,2);
    return sqrt(snon2+sbohr2);
}

//////////////////////////////////////////////////////////////////

double MainWindow::straggling(int elem, double energy, double sigma)
{
    double Esum_gauss, cross_gauss, cross_sum;
    int dE_strag;

    cross_sum = 0;
    Esum_gauss = energy - 3*sigma;
    dE_strag = 1 ;           ////// Straggling integration step
    while (Esum_gauss < energy + 3*sigma)
    {
           cross_gauss = interpol(elem,energy);
           cross_sum += cross_gauss*(1/(sigma*sqrt(2*3.1415)))*exp(-pow((Esum_gauss-energy),2)/(2*pow(sigma,2)));
           Esum_gauss += dE_strag;
     }
     return cross_sum;
}

//////////////////////////////////////////////////////////////////

void MainWindow::show_selected()
{
    int i = element_combo->findText(targetList->item(targetList->currentRow())->text()
                                    .left(2).trimmed(),Qt::MatchExactly);
  element_combo->setCurrentIndex(i);
  atomic_perc_spin->
         setValue(targetList->item(targetList->currentRow())->text().remove(0,6).left(6).trimmed().toDouble());
  if (program_choice == 2)
  {
      i = behav_combo->findText(targetList->item(targetList->currentRow())->text()
                             .right(5).trimmed(),Qt::MatchExactly);
        behav_combo->setCurrentIndex(i);
  }
}

//////////////////////////////////////////////////////////////////
void MainWindow::open_r33()
{
    QString dummy;
    open_r33_file(dummy);
}

void MainWindow::open_r33_file(QString fileName)
{

        QFile r33filename;
        QTextStream in(&r33filename);
        QStringList list, Z_list;
        QString Z_str, Egamma_str, Theta_str, Reaction_str, Source_str, En_str, ExtrMass_str, Units_str;
        QString Z_str_dir,Z_str_elem;
        QString dummy;
        std::string dummy1;
        std::map<double,double>m;
        bool data_begin, Z_exist;
        bool ok;
        double nada, cross_fact;

        E_cross.clear();
        Cross_section.clear();

        r33filename.setFileName(fileName);
        r33filename.open(QFile::Text|QFile::ReadOnly);
        data_begin = false;
        Z_exist = false;

         unsigned int k=0;
         while(!in.atEnd())
         {

                     dummy = in.readLine();
                     if(dummy.left(8) == "EndData:")
                     {
                         break;
                     }
                     if( dummy.left(5) == "Zeds:")
                     {                   
                         Z_list = Reaction_str.trimmed().split(QRegExp("\\("));
                         dummy1 = Z_list[0].right(2).left(1).toStdString();
                            if(isdigit( dummy1[0]) == false)
                             Z_str_elem = Z_list[0].right(2);
                         else
                             Z_str_elem = Z_list[0].right(1);
   //                       qDebug() << Z_str_elem <<" " << Reaction_str;
                         Z_str_elem = find_element_by_name(Elements,Z_str_elem)+"  "; //// Find Z from reaction

                        list = dummy.remove(0,5).split(QRegExp(","));
                        if(list[0] != " ")
                        {
                           Z_str_dir = list[1].trimmed() +"  ";    //// Find Z from line if any
                           Z_exist = true;
                        }
                        if (Z_exist == true)
                        {    if(QString::compare(Z_str_dir, Z_str_elem, Qt::CaseInsensitive) == 0) /// Sanity check for Z if both Z's present
                               Z_str = Z_str_dir;
                           // else
                           //   warning_code(1);
                         }
                         else
                            Z_str = Z_str_elem; /// Otherwise proceed with reaction Z

   //                     qDebug() << Z_str_elem <<" "<<Z_str_dir;
                         list.clear();
                     }
                      if( dummy.left(10) == "Enfactors:")
                     {
                        list = dummy.remove(0,10).trimmed().split(QRegExp(","));
                        En_str = list[0].trimmed();
                        if(En_str.toFloat() == 0 ) En_str = "1";
                        list.clear();
                     }

                    if( dummy.left(7) == "Egamma:") Egamma_str = dummy.remove(0,7).trimmed() +"  ";
                    if( dummy.left(6) == "Theta:") Theta_str = dummy.remove(0,6).trimmed() +"  ";
                    if( dummy.left(9) == "Reaction:") Reaction_str = dummy.remove(0,9).trimmed() +"  ";
                    if( dummy.left(7) == "Source:") Source_str = dummy.remove(0,7).trimmed() +"  ";
                    if( dummy.left(6) == "Units:") Units_str = dummy.remove(0,6).trimmed();

                    cross_fact = 1;
                    ExtrMass_str = Reaction_str.trimmed().left(2);
                    if (ExtrMass_str.toFloat() == 0 ) ExtrMass_str = Reaction_str.trimmed().left(1);

                    if( Units_str=="tot" ) cross_fact = 4*3.1415;
                    if( Units_str=="yield" ) { error_code(15,""); return;}

                    if(data_begin)
                       {
                        int check_in=0;
                        list = dummy.trimmed().split(QRegExp ("\t|,| "));
                        for(int kk=0; kk < list.size();++kk)
                        {
                          nada = list[kk].trimmed().toDouble(&ok);
                          if(ok)
                           {
                              if (check_in == 0) E_cross.push_back(nada*En_str.toFloat());
                              if (check_in == 2) Cross_section.push_back(nada/cross_fact);
                              check_in++;
                           }
                         }
                        list.clear();
                        k++;
                        }

                   if(dummy.left(5) == "Data:")
                      data_begin = true;

          }
          r33filename.close();

         /////  In case where the Energies are in descending order /////
         if( E_cross[0] > E_cross[E_cross.size()-1])
         {
          for (size_t i = 0; i < E_cross.size(); ++i)
              m[E_cross[i]] = Cross_section[i];

          E_cross.clear();
          Cross_section.clear();
          for (std::map<double,double>::iterator it=m.begin(); it!=m.end(); ++it)
            {
              E_cross.push_back(it->first);
              Cross_section.push_back(it->second);
            }
         }

         /////////////////////////////////////////////////////////////
          r33_info.clear();
          r33_info.push_back(Z_str);
          r33_info.push_back(ExtrMass_str);
          r33_info.push_back(Egamma_str);
          r33_info.push_back(Theta_str);
          r33_info.push_back(Reaction_str);
          r33_info.push_back(Source_str);
          r33_info.push_back(QString::number(k));

}

//////////////////////////////////////////////////////////////////


//////////////////        Runs when Create Reaction List is pressed  /////////////////////////

void MainWindow::create_reaction_file()
{
    std::string filepath;
    filepath = QApplication::applicationDirPath().toStdString().c_str();
    if(filepath=="/usr/local/bin") filepath=(QDir::homePath()+"/.R33Data/reactions.dat").toStdString().c_str();
    else filepath="./R33Data/reactions.dat";
    QFile out_file(QString::fromStdString(filepath));
    out_file.open(QIODevice::WriteOnly | QIODevice::Text);
    QTextStream out(&out_file);
    filepath = QApplication::applicationDirPath().toStdString().c_str();
    if(filepath=="/usr/local/bin") filepath=(QDir::homePath()+"/.R33Data").toStdString().c_str();
    else filepath="./R33Data";
    QDirIterator dirIt(QString::fromStdString(filepath), QDirIterator::Subdirectories);
    while (dirIt.hasNext())
    {
        dirIt.next();
        if (QFileInfo(dirIt.filePath()).isFile())
            if (QFileInfo(dirIt.filePath()).suffix() == "r33" || QFileInfo(dirIt.filePath()).suffix() == "R33")
            {

              open_r33_file(dirIt.filePath());
              out << r33_info[0] + r33_info[1] +" " + r33_info[2] + r33_info[3] + r33_info[6]+" " +QString::number(E_cross[0])+" " +
                     QString::number(E_cross[r33_info[6].toFloat()-1])+" " + r33_info[4] + dirIt.fileName() +" "+ r33_info[5] << endl;
             }

    }
 out_file.close();
}
////////////////////////////////////////////////////////////////////

void MainWindow::save_results()
{
        QFile savefile;
        QTextStream out(&savefile);

        QString fileName = QFileDialog::getSaveFileName(0,"Save result file",".",tr("dat file (*.dat)"));

                  savefile.setFileName(fileName);
                  savefile.open(QFile::Text|QFile::WriteOnly);

                out << ResultPane->toPlainText();
                savefile.flush();
                savefile.close();
    }
//////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////

void MainWindow::clear_results()
{

                ResultPane->clear();
                //savefile.flush();
                //savefile.close();
    }
//////////////////////////////////////////////////////////////////
    void MainWindow::build_Target()
    {

//        if (target_window==0) target_window = new TargetWindow(this);
//        qDebug()<<target_window->isVisible();

        if(!target_window->isVisible())
   {        target_window->show();

    }
       else
            target_window->activateWindow();

        QObject::connect(target_window,SIGNAL(sendTarget(std::vector < target_element >)),
                                      this,SLOT(getTarget(std::vector < target_element >)));
        }
//////////////////////////////////////////////////////////////////


double calc_stop_power(int Z1, int Z2, double mass, double E)
{
    QFile stopFile;
    QTextStream in(&stopFile);
    QString dummy;
    QStringList list;
    unsigned int x_points=0;
    int z1_current,z2_current;
    double x_min=0,x_max=0;
    std::vector <double> params;
    std::string filepath;
    double gamma,x,i_float;
    double C_C2  = 8.9875518e+16;
    double C_KEV = 1.6021917e-16;
    double C_AMU = 1.66044e-27;
    filepath = QApplication::applicationDirPath().toStdString().c_str();
   if(filepath=="/usr/local/bin") filepath="/usr/local/share/PiGreCo/data/srim.txt";
   else filepath.append("/data/srim.txt");
    stopFile.setFileName(filepath.c_str());
    stopFile.open(QFile::Text|QFile::ReadOnly);

       while(!in.atEnd())
       {
                   dummy = in.readLine();

                   if(dummy.left(6) == "x-min=")
                   {
                       list = dummy.trimmed().split(QRegExp("="));
                       x_min = list[1].toDouble();
                   }
                   if(dummy.left(6) == "x-max=")
                   {
                       list = dummy.trimmed().split(QRegExp("="));
                       x_max = list[1].toDouble();
                   }
                   if(dummy.left(9) == "x-points=")
                   {
                       list = dummy.trimmed().split(QRegExp("="));
                       x_points = list[1].toInt();
                   }
                   if(dummy.left(9) == "#STOPPING")
                   {
                      list = dummy.trimmed().split(QRegExp("="));
                      z1_current = list[1].left(2).trimmed().toInt();
                      z2_current = list[2].toInt();
                      //qDebug() << z1_current << z2_current;
                      if (Z1 == z1_current && Z2 == z2_current)
                          for (unsigned it=0; it < x_points;it++)
                          {
                              dummy = in.readLine();
                              params.push_back(dummy.trimmed().toDouble());
                          }
                   }

        }

       mass = mass * 1.66044e-33 ; //(amu)
       gamma=1.0+E/(mass*C_C2);
       x=(gamma-1)*C_C2/(C_KEV/C_AMU);
       i_float = (log10(x)-log10(x_min))/(log10(x_max)-log10(x_min)) * (x_points-1);
       int i =  floor(i_float);
       return ((params[i+1]-params[i])*(i_float-1.0*i))+params[i];
}
//////////////////////////////////////////////////////////////////

    double MainWindow::energy_stop_calc(double En)
    {
        double E;
        char *target_name;
        int Zstop[100], Mass[100];
        int Z;
        double Density[100], dummy, A1[100],A2[100],A3[100],A4[100],A5[100],A6[100],A7[100],A8[100];
        double slow,shigh;
        std::string line, dummy1;

        unsigned int i=0;
        double atom_perc;
        double sum_atomic = 0;
        double sum_stop = 0;
        std::ifstream stopfile;

       if(srimAct->isChecked()==true)
     {

           E =En*1.6021917e-16;
           isotope incident;
           std::vector < isotope > list_of_isotopes;

            list_of_isotopes = create_isotopes();
            incident =find_isotope_by_name(list_of_isotopes,"H:1");

            for(unsigned int i=0;i<=Target.size()-1;i++)
             {

             std::vector<char> writable(Target[i].atom.Element_Name.begin(),
                                        Target[i].atom.Element_Name.end());
              writable.push_back('\0');
              target_name = &writable[0];

             atom_perc = Target[i].atomic_percent;

             Z=find_isotope_by_Z(list_of_isotopes,target_name);
             sum_atomic+=atom_perc;
             sum_stop += calc_stop_power(incident.Atomic_Number,Z,incident.Mass,E)*atom_perc;
        }
       }

       if(zblAct->isChecked()==true)
       {
           dummy1 = QApplication::applicationDirPath().toStdString().c_str();
   if(dummy1=="/usr/local/bin") dummy1="/usr/local/share/PiGreCo/data/SCOEF.95A";
   else dummy1.append("/data/SCOEF.95A");

               stopfile.open(dummy1.c_str());

               getline(stopfile,line);
               getline(stopfile,line);
               while(stopfile >> Zstop[i]>>Mass[i]>>dummy>>Density[i]>>dummy>>dummy>>dummy>>dummy>>A1[i]>>A2[i]>>A3[i]
                     >>A4[i]>>A5[i]>>A6[i]>>A7[i]>>A8[i])
               {
                     i++;
               }

               double sn=1;
               En = En/1.0078;
               for(unsigned int k=0; k <= Target.size()-1;k++)
               {
                   atom_perc = Target[k].atomic_percent;
                   for( unsigned j=0; j < i; j++)
                   {
                       if(Target[k].atom.Atomic_Number == Zstop[j])
                       {
                           slow = A1[j]*pow(En,A2[j]) + A3[j]*pow(En,A4[j]);
                           shigh = A5[j]/pow(En,A6[j])*log(A7[j]/En+A8[j]*En);
                           sn = slow*shigh/(slow+shigh);
                       }
                   }
                   sum_atomic+= atom_perc;
                   sum_stop += sn*atom_perc;
                 }
       }
       return sum_stop/sum_atomic;

     }

    //////////////////////////////////////////////////////////////////

///////////////    Step Dialogue Functions ///////////////////////////////////////////////////////////////////
/// \brief stepDialog::stepDialog
/// \param parent
///
///
///  Constructor
///
///

stepDialog::stepDialog(QWidget *parent)
      : QDialog(parent)
  {
  //  step inter;
    setWindowFlags(this->windowFlags() &  ~Qt::WindowContextHelpButtonHint & ~Qt::WindowTitleHint);
    minStep_label = new QLabel();
    minStep_label->setMinimumHeight(22);
    minStep_label->setMaximumHeight(22);
    minStep_label->setText("Enter new step");

    minStep_spin = new QDoubleSpinBox;
    minStep_spin->setRange(0,100);

    step_check = new QCheckBox;
    step_check->setHidden(false);
    connect(step_check, SIGNAL(toggled(bool)), this, SLOT (StepState() ));
    step_check->setCheckState(Qt::Checked);

    step_label = new QLabel();
    step_label->setText("  Default Step");
    step_label->setHidden(false);
    step_label->setAlignment(Qt::AlignCenter);

    closeButton = new QPushButton(tr("Close"));
    connect(closeButton, SIGNAL(released()), this, SLOT(StepClose()));

     QGridLayout *layout = new QGridLayout;
     layout->addWidget(step_label,0,0);
     layout->addWidget(minStep_label,1,0);
     layout->addWidget(step_check,0,1);
     layout->addWidget(minStep_spin,1,1);
     layout->addWidget(closeButton,2,0,2,0);
     setLayout(layout);

}
void stepDialog::StepState()
{
     minStep_label->setDisabled(step_check->checkState());
     minStep_spin->setDisabled(step_check->checkState());
}


void stepDialog::StepClose()
  {
    close();
}

void stepDialog::SetData(double step)
{
  minStep_spin->setValue(step);
}
step stepDialog::GetData()
{
inter.stepstate = step_check->checkState();
inter.stepvalue = minStep_spin->value();
  return inter;
}
void MainWindow::closeEvent(QCloseEvent *event)
{
    if(target_window->isVisible()) target_window->close();
    event->accept();
    QCoreApplication::quit() ;

}
void MainWindow::createStatusBar()
{

    statusLabel = new QLabel();
         statusLabel->setText("Ready");
           statusBar()->addWidget(statusLabel);
 //  statusBar();
  // statusBar()->showMessage(tr("Ready"),1000);
 //           setWindowModified(true);
}
